import numpy as np

class UnconstrainedProgram:

    def get_init_theta(self):
        """
        Returns an initial parameter vector (theta).

        Returns:
        - theta (np.ndarray): Initial parameter vector.
        """
        pass

    def objective(self, theta):
        """
        Computes the objective function (to minimize) for the given parameters.

        Args:
        - theta (np.ndarray): Parameter vector.

        Returns:
        - loss (float): Objective function value, f(theta).
        """
        pass
    
    def gradient(self, theta):
        """
        Computes the gradient of the objective function with respect to the parameters.

        Args:
        - theta (np.ndarray): Parameter vector.

        Returns:
        - gradient (np.ndarray): Gradient vector.
        """
        pass
    
    def hessian(self, theta):
        """
        Computes the Hessian of the objective function with respect to the parameters.

        Args:
        - theta (np.ndarray): Parameter vector.

        Returns:
        - hessian (np.ndarray): Hessian matrix.
        """
        pass

class LogisticRegression(UnconstrainedProgram):
    """
    Logistic regression for binary classification.
    """
    
    def __init__(self, X_train, y_train, eps=1e-3):
        """
        Initializes a binary logistic regression model.

        Args:
        - input_size (int): The number of input features.
        """
        self.input_size = X_train.shape[1]
        self.X_train = X_train
        self.y_train = y_train
        self.eps = eps

    def get_init_theta(self):
        theta = np.zeros(self.input_size)
        return theta

    def _sigmoid(self, z):
        """
        Computes the sigmoid function elementwise for a given vector z (np.array).
        """
        return 1 / (1 + np.exp(-z) + self.eps)

    def objective(self, theta):
        x, y = self.X_train, self.y_train
        prob = self.probability(theta, x)
        return - (y * np.log(prob) + (1 - y) * np.log(1 - prob)).mean()
    
    def gradient(self, theta):
        x, y = self.X_train, self.y_train
        prob = self.probability(theta, x)
        gradient = x.T.dot(prob - y) / x.shape[0]
        return gradient
    
    def hessian(self, theta):
        x, y = self.X_train, self.y_train
        prob = self.probability(theta, x)
        diag = np.diag(prob * (1 - prob))
        hessian = x.T.dot(diag).dot(x) / x.shape[0]
        return hessian
    
    def probability(self, theta, X_set):
        """
        Computes the probability of the positive class (y=1) for the given feature vectors and parameters.

        Args:
        - theta (np.ndarray): Parameter vector.
        - X_set (np.ndarray): Collection of feature vectors.

        Returns:
        - prob (float or np.ndarray): Probability of the positive class p(y=1 | x, theta) for each feature vector.
        """
        z = X_set.dot(theta)
        prob = self._sigmoid(z)
        return prob
    
    def accuracy(self, theta, X_set, y_set):
        """
        Computes the accuracy of the model on the given evaluation dataset of (x,y) pairs.

        Args:
        - theta (np.ndarray): Parameter vector.
        - X_set (np.ndarray): Collection of feature vectors.
        - y_set (np.ndarray): Corresponding ground truth labels for each feature vector in the X_set collection.

        Returns:
        - accuracy (float): Accuracy.
        """
        prob = self.probability(theta, X_set)
        y_pred = prob >= 0.5
        return (y_pred == y_set).mean()



class ConstrainedProgram:

    def get_init_theta(self):
        """
        Returns an initial parameter vector (theta).

        Returns:
        - theta (np.ndarray): Initial parameter vector.
        """
        pass

    def cp_objective_func(self, theta):
        """
        Computes the objective function (to minimize) for the given parameters.

        Args:
        - theta (np.ndarray): Parameter vector.

        Returns:
        - loss (float): Objective function value, f(theta).
        """
        pass

    def cp_constraint_func(self, theta):
        """
        Computes the left-hand-side of the inequality constraints A (theta) - b <= 0.

        Args:
        - theta (np.ndarray): Parameter vector.

        Returns:
        - A(theta)-b (np.array): The value of the constraints.
        """
        pass

class Simple2DProgram(ConstrainedProgram):
    """
    2D mathematical program with linear constraints.
    """
    
    def __init__(self, initial_point):
        """
        Initializes a binary logistic regression model.

        Args:
        - initial_point (np.array): The initial point for optimization.
        """
        assert len(initial_point) == 2
        self.input_size = 2
        self.initial_point = initial_point
        # TODO: implement this

    def get_init_theta(self):
        return self.initial_point

    def cp_objective_func(self, theta):
        # TODO: implement this    
        
    def cp_constraint_func(self, theta):
        # TODO: implement this
    
    def objective_barrier(self, theta, t):
        # TODO: implement this
        
    def gradient_barrier(self, theta, t):
        # TODO: implement this
    
    def hessian_barrier(self, theta, t):
        # TODO: implement this
    