import numpy as np
import matplotlib.pyplot as plt

def plot_log_log_convergence(loss_list, label_list, xlab, ylab, title, mypath=None, logscale_x=True, logscale_y=True):
    '''
    Plotting loss arrays with corresponding labels

    loss_list: 2d nested list of losses (each elem corresponds to a different loss curve)
    label_list: 1d list of string labels to give different curves
    xlab: x-axis label
    ylab: y-axis label
    title: plot title
    mypath: Path to output file (optional)
    '''
    assert(len(loss_list) == len(label_list))

    mpl_colors = ['red', 'blue', 'green', 'red', 'purple', 'brown', 'pink', 'gray', 'olive', 'cyan']

    for i, losses in enumerate(loss_list):
        iters = list(range(len(losses)))
        plt.plot(iters, losses, color=mpl_colors[i], label=label_list[i])

    if logscale_x:
        plt.xscale('log')
    if logscale_y:
        plt.yscale('log')
    plt.xlabel(xlab)
    plt.ylabel(ylab)
    plt.title(title)
    plt.legend()
    if mypath is not None:
        plt.savefig(mypath)
    plt.show()
    
def plot_objective_constraints_iterates(iterates, mypath=None):
    '''Plots the objective function and constraints for the small 2D mathematical program used by HW4's Log-Barrier Method implementation. Also plots the iterates from a single run of the Log-Barrier method.
    
    Args:
      iterates (2D numpy.array OR list of 1D np.arrays): assumes the iterates are stored in the form [[x_1^0, x_2^0], [x_1^1, x_2^1], ..., [x_1^T, x_2^T]]
    '''
    # Define the objective function
    def f(x1, x2):
        min1 = 0
        min2 = -30
        return (10 * (x1-min1)**2 + (x2-min2)**2) / 2  + 50 * np.log(1 + np.exp(-(x1-min1/2) - (x2-min2/2)))

    # Create a grid of x1 and x2 values
    x1 = np.linspace(-20, 20, 400)
    x2 = np.linspace(-20, 20, 400)
    X1, X2 = np.meshgrid(x1, x2)
    Z = f(X1, X2)

    # Create a contour plot
    plt.figure(figsize=(8, 8))
    contours = plt.contour(X1, X2, Z, levels=30, colors='b')
    plt.clabel(contours, inline=True, fontsize=8)
    plt.xlabel('x1')
    plt.ylabel('x2')
    plt.title('Contour Plot of f(x1, x2)')
    plt.grid()

    # Plot the five inequalities by showing their corresponding lines
    x1_values = np.linspace(-20, 20, 100)

    # Line 1 (Bottom side of the pentagon)
    x2_1 = 0.4*x1_values - 10
    plt.plot(x1_values, x2_1, label='Line 1: $x_2 = 1$')

    # Line 2 (Top left side of the pentagon)
    x2_2 = 0.5 * x1_values + 15
    plt.plot(x1_values, x2_2, label='Line 2: $0.5x_1 + 1.5$')

    # Line 3 (Top right side of the pentagon)
    x2_3 = -x1_values + 15
    plt.plot(x1_values, x2_3, label='Line 3: $-x_1 + 3$')

    # Line 4 (Bottom Left side of the pentagon)
    x2_4 = -3 * x1_values - 30
    plt.plot(x1_values, x2_4, label='Line 4: $-0.5x_1 + 2.5$')

    # Line 5 (Bottom right side of the pentagon)
    x2_5 = 2*x1_values - 30
    plt.plot(x1_values, x2_5, label='Line 5: $x_1 - 1$')

    # Set the axis limits
    plt.xlim(-20, 20)  # Set the x-axis limits
    plt.ylim(-20, 20)  # Set the y-axis limits

    x1_points = [x1 for x1,x2 in iterates]
    x2_points = [x2 for x1,x2 in iterates]
    plt.scatter(x1_points, x2_points, color='red')
    plt.plot(x1_points, x2_points, color='red')

    if mypath is not None:
        plt.savefig(mypath)
    plt.show()
