import numpy as np

from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler

from models import *
from optimization import *
from plot import *

def get_data(standardize=True):
    """
    Load the Breast Cancer dataset and split it into training and testing sets.

    Args:
    - standardize (bool, optional): Whether to standardize the features (default is True), by centering them around mean=0 and scaling to std=1.
    
    Returns:
    - X_train (np.ndarray): Training features.
    - y_train (np.ndarray): Training labels.
    - X_test (np.ndarray): Testing features.
    - y_test (np.ndarray): Testing labels.

    Example:
    X_train, y_train, X_test, y_test = get_data(standardize=True)
    """
    # Load the Breast Cancer dataset
    data = load_breast_cancer()
    X = data.data
    X = np.concatenate([np.ones((X.shape[0], 1)), X], axis=1)
    y = data.target
    
    # Split the dataset into training and testing sets
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)
    
    if standardize:
        # Standardize the features (mean=0, std=1)
        scaler = StandardScaler()
        X_train = scaler.fit_transform(X_train)
        X_test = scaler.transform(X_test)
    
    return X_train, y_train, X_test, y_test

def train_and_test_by_name(model_name, epsilon, mu=None, initial_point=None, t0=None):
    print(f'Model: {model_name}')

    # Initialize model
    if model_name == 'lr':
        X_train, y_train, X_test, y_test = get_data(standardize=True)
        model = LogisticRegression(X_train, y_train)
        opt = NewtonsMethod(num_epochs=100, epsilon=epsilon, tau0=1.0, alpha=0.7, beta=0.9)
        # Train and test
        loss, acc = opt.optimize(model, X_test, y_test)
        print(f"Test Accuracy: {acc[-1]:.4f}")
        return loss, acc
    else:
        model = Simple2DProgram(initial_point=initial_point)
        opt = LogBarrierMethod(mu=mu, t0=t0, delta=1e-2, num_epochs=100, epsilon=epsilon, tau0=1.0, alpha=0.7, beta=0.9)
        loss, points, dualgap = opt.optimize(model)
        return loss, points, dualgap
    
def main():
    # TODO: implement this
    
if __name__ == '__main__':
    main()